///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2023 Smaser AG
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Sep 21, 2023	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.control;

import java.util.ArrayList;
import java.util.List;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbControlObject;
import com.tietoenator.trip.jxp.TdbControlObjectType;
import com.tietoenator.trip.jxp.control.TdbDatabaseList;
import com.tietoenator.trip.jxp.control.TdbControlObjectList;
import com.tietoenator.trip.jxp.database.TdbDatabaseAccess;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.session.TdbControlCacheManager;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;


/**
 * Acquire CONTROL database information, with and without caching.
 * Illustrates usage difference and potential performance boosts.
 */
public class Caching
{
    private TdbSession m_session;
    private long duration_nocache_list = 0;
    private long duration_nocache_designs = 0;
    private long duration_cache_preload = 0;
    private long duration_cache_list = 0;
    private long duration_cache_designs = 0;

    Caching(TdbSession session)
    {
        m_session = session;
    }

    void withoutCaching(int laps, boolean emit_log) throws TdbException
    {
        TdbDatabaseList dblist = null;
        long checkpoint1,checkpoint2;

        checkpoint1 = System.currentTimeMillis();
        for (int i=0;i<laps;i++)
        {
            dblist = new TdbDatabaseList(m_session);
        }
        duration_nocache_list = System.currentTimeMillis() - checkpoint1;
        if (emit_log)
            System.out.println(String.format("Database list: %d ms in total and %.03f ms in average",duration_nocache_list,(double)duration_nocache_list/(double)laps));


        ArrayList<TdbDatabaseDesign> designs = new ArrayList<TdbDatabaseDesign>();
        checkpoint2 = System.currentTimeMillis();
        for (int i=0;i<laps;i++)
        {
            for (TdbControlObject ctlobj : dblist)
            {
                if (ctlobj.isDatabase() || ctlobj.isThesaurus())
                {
                    TdbDatabaseDesign design = new TdbDatabaseDesign(m_session,ctlobj);
                    design.get();
                    designs.add(design);
                }
            }
            designs.clear();
        }
        duration_nocache_designs = System.currentTimeMillis() - checkpoint2;
        if (emit_log)
            System.out.println(String.format("Database designs: %d ms in total and %.03f ms in average",duration_nocache_designs,(double)duration_nocache_designs/(double)laps));
    }
	
    void withCaching(int laps) throws TdbException
    {
        long checkpoint0, checkpoint1, checkpoint2;
        TdbDatabaseList dblist = null;

        TdbControlCacheManager cache = m_session.getControlCache();

        checkpoint0 = System.currentTimeMillis();
        cache.load();
        duration_cache_preload = System.currentTimeMillis() - checkpoint0;
        System.out.println(String.format("Preloading time: %d ms ",duration_cache_preload));

        checkpoint1 = System.currentTimeMillis();
        for (int i=0;i<laps;i++)
            dblist = cache.getDatabaseList();
        duration_cache_list = System.currentTimeMillis() - checkpoint1;
        System.out.println(String.format("Database list: %d ms in total and %.03f ms in average",duration_cache_list,(double)duration_cache_list/(double)laps));

        ArrayList<TdbDatabaseDesign> designs = new ArrayList<TdbDatabaseDesign>();
        checkpoint2 = System.currentTimeMillis();
        for (int i=0;i<laps;i++)
        {
            for (TdbControlObject ctlobj : dblist)
            {
                if (ctlobj.isDatabase() || ctlobj.isThesaurus())
                {
                    TdbDatabaseDesign design = cache.getDatabaseDesign(ctlobj.getName());
                    if (design != null) 
                        designs.add(design);
                }
            }
            designs.clear();
        }
        duration_cache_designs = System.currentTimeMillis() - checkpoint2;
        System.out.println(String.format("Database designs: %d in total and %.03f ms in average",duration_cache_designs,(double)duration_cache_designs/(double)laps));
    }

    void summary(int laps)
    {
		long cache_designs_time = duration_cache_preload + duration_cache_designs;
		long cache_list_time = duration_cache_preload + duration_cache_list;
		
        long gain_list = (duration_nocache_list - cache_list_time) / (long)laps;
        long gain_designs = (duration_nocache_designs - cache_designs_time) / (long)laps;

        long times_list = duration_nocache_list / (0 == cache_list_time ? 1 : cache_list_time);
        long times_designs = duration_nocache_designs / (0 == cache_designs_time ? 1 : cache_designs_time);

        System.out.println();
        if (gain_list > 0)
		{
            System.out.print(String.format("List gain: %d ms",gain_list));
			if (times_list > 1)
				System.out.println(String.format(" - %d times faster",times_list));
			else
				System.out.println(" - no significant improvement");
		}
        else
            System.out.println("List gain: no improvement");
		
        if (gain_designs > 0)
		{
            System.out.print(String.format("Designs gain: %d ms",gain_designs));
			if (times_designs > 1)
				System.out.println(String.format(" - %d times faster",times_designs));
			else
				System.out.println(" - no significant improvement");
		}
        else
            System.out.println("Designs gain: no improvement");
    }

    void showInfo() throws TdbException
    {
        List<String> names;
        TdbControlCacheManager cache = m_session.getControlCache();

        System.out.println();
        System.out.println("DATABASES:");
        for (String name : cache.getDatabaseNames())
            System.out.println("  "+name);

        System.out.println("THESAURI:");
        for (String name : cache.getThesaurusNames())
            System.out.println("  "+name);

        System.out.println("CLUSTERS:");
        for (String name : cache.getClusterNames())
            System.out.println("  "+name);
    }

    void clearCache()
    {
        m_session.getControlCache().clear();

    }

   /**
    * Acquire CONTROL database information, with and without caching.
    * Illustrates usage difference and potential performance boosts.
    * @param args Not used
    */
   public static void main(String[] args)
   {
      TdbControlObjectList list;
      TdbSession session = null;
      Console stdin = new Console();
      int laps = 250;

      try
      {
         // Retrieve the appropriate type of list iterator depending on session type
         session = SessionFactory.newInstance("**** CONTROL object caching ****", true);

         Caching app = new Caching(session);
         app.showInfo();
         app.clearCache();
         app.withoutCaching(1,false);

         System.out.println();
         System.out.println(String.format("Running tests with %d laps per test",laps));

         System.out.println();
         System.out.println("Without caching:");
         app.withoutCaching(laps,true);

         System.out.println();
         System.out.println("With caching:");
         app.withCaching(laps);

         app.summary(laps);
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
      finally
      {
        if (session != null)
        {
            try { session.logout(); } catch (Exception ignore) {}
        }
      }
   }

}

