///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2013 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//    TF May 24, 2013   Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.*;
import com.tietoenator.trip.jxp.data.*;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.database.TdbFieldDesign;
import com.tietoenator.trip.jxp.logging.*;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.io.FileInputStream;
import java.util.ArrayList;

/**
 * TRIPjxp example program that illustrates how TRIPcof can be used to import
 * a document, extract text from it and to annotate the record with keywords 
 * and metadata.
 */
public class CofAnalyze
{
    private TdbSession m_session;
    private String m_database = null;
    private String m_stringField = null;
    private String m_textField = null;
    private String m_pnameField = null;
    private String m_pvalueField = null;
    private String m_keywordField = null;
    private String m_mnameField = null;
    private String m_mvalueField = null;
    private ArrayList<String> m_allOutputFields = new ArrayList<String>();

    /**
     * Constructor
     * @throws TdbException
     */
    public CofAnalyze()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIPcof Document Analysis Example ****", false);
    }

    void close()
    {
        if (m_session != null)
        {
            try { m_session.logout(); } catch (Exception ignore) {}
            m_session = null;
        }
    }

    private void setDatabase(String db)
    {
        m_database = db;
    }

    private void setStringField(String sf)
    {
        m_stringField = sf;
    }

    private void setTextField(String tf)
    {
        m_textField = tf;
    }

    private void setPNameField(String pn)
    {
        m_pnameField = pn;
        m_allOutputFields.add(pn);
    }

    private void setPValueField(String pn)
    {
        m_pvalueField = pn;
        m_allOutputFields.add(pn);
    }

    private void setMNameField(String pn)
    {
        m_mnameField = pn;
        m_allOutputFields.add(pn);
    }

    private void setMValueField(String pn)
    {
        m_mvalueField = pn;
        m_allOutputFields.add(pn);
    }

    private void setKeywordField(String pn)
    {
        m_keywordField = pn;
        m_allOutputFields.add(pn);
    }

    /**
     * Perform text extraction and analysis
     */
    public void process()
            throws Exception
    {
        TdbRecord rec = null;

        Console stdin = new Console();
        String name = null;
        while (name == null)
            name = stdin.readln("Name of file to extract text from", null, true);

        String namefield = null;
        while (namefield == null)
            namefield = stdin.readln("Field for file name", "FILE_NAME", true);


        rec = new TdbRecord(m_session, m_database, false);

        TdbTextField fld = (TdbTextField) rec.getHead().createField(m_textField, TdbFieldType.TextField);
        TdbTextExtractionInfo txi = fld.getTextExtractionInfo();

        // The optional string field into which the file is to be stored.
        if (m_stringField != null)
        {
            txi.setBinaryCopyField(m_stringField);
        }

        // The optional phrase field that receieves property name/value pairs.
        if (m_pnameField != null && m_pvalueField != null)
        {
            txi.setPropertyNameField(m_pnameField);
            txi.setPropertyValueField(m_pvalueField);
        }

        // Enable client-side extraction?
        if (0 != stdin.readint("Client-side Extraction (0=no, 1=yes)", 0, true))
            txi.setClientSide(true);

        // In contrast to TRIPview and TRIPview-C, the name is an extremely important 
        // peice of information. You must always supply the file name, even if the file
        // to be processed is only present in memory! Also, you are recommended
        // to store the name of the file into the record as well for future
        // reference (e.g. for use by TRIPcof HTML conversion).
        txi.setFileName(name);
        rec.getHead().createField(namefield, TdbFieldType.PhraseField).appendValue(name);

        // Read file data using a stream? Note that if client-side operation is
        // performed, the FileName property (as set above) is sufficient.
        // Passing file data via a stream to JXP should only be done if
        //   1) the file is in memory only, or
        //   2) if TRIPcof and the application are running on two different
        //      machines.
        if (txi.getClientSide() == false)
            txi.setStream(new FileInputStream(name));

        // Remember to explicitly state that text extraction is to be performed.
        txi.setExtractText(true);

        TdbNlpInfo nlp = rec.getNlpInfo();
        nlp.addInputField(m_textField);

        if (stdin.readbool("Keyword analysis", true, true))
        {
             String kwfld = null;
             while (kwfld == null)
             {
                kwfld = stdin.readln("Keyword output field", "KEYWORDS");
                
                if (kwfld != null)
                {
                    nlp.setKeywordField(kwfld);
                    nlp.setExtractKeywords(true);
                    setKeywordField(kwfld);
                }
             }
        }
        
        if (stdin.readbool("Metadata analysis", true, true))
        {
            nlp.setFieldMapping(TdbNlpInfo.FieldMappingType.Tuples);

            String tn=null,tv=null;
            do
            {
                tn = stdin.readln("Phrase field for category names","META_NAME");
                if (tn == null)
                {
                    System.out.println("Please specify a phrase field!");
                    tn = null;
                }
            }
            while (null == tn);
            do
            {
                tv = stdin.readln("Phrase field for category values","META_VALUE");
                if (tv != null && tv.equalsIgnoreCase(tn))
                {
                    System.out.println("Please specify a different phrase field!");
                    tv = null;
                }
            }
            while (null == tv);

            nlp.setMetadataTupleFields(tn,tv);
            nlp.setExtractMetadata(true);
            setMNameField(tn);
            setMValueField(tv);
        }


        // Send the request to the server.
        rec.commit();


        System.out.println();
        System.out.println("Document successfully stored in record #"+
                Integer.toString(rec.getRecordId()));

        if (m_allOutputFields.size()>0)
        {
            // Retrieve the output fields for the record
            for (String fieldname : m_allOutputFields)
                rec.addToTemplate(fieldname);
            rec.get();

            System.out.println("Analysis results are:\n");
        
            TdbComponent h = rec.getHead();
            for (String fieldname : m_allOutputFields)
            {
                TdbField ofld = h.getField(fieldname);
                System.out.print(ofld.getName() + ": ");
                for (int i=0;i<ofld.getValueCount();i++)
                {
                    if (i>0)
                        System.out.print(", ");
                    System.out.print(ofld.getValue(i));
                }
                System.out.println("\n");
            }
        }
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        CofAnalyze program = null;
        try
        {
            program = new CofAnalyze();
            String db = null, txt = null, pname = null, pval = null;
            Console stdin = new Console();

            // Get a valid database name, and set the list of
            // retrieval fields from it
            while (db == null)
                db = stdin.readln("TRIPcof Sample Database", null);

            while (txt == null)
                txt = stdin.readln("Text field", "FILE_TEXT", true);

            program.setDatabase(db);
            program.setTextField(txt);

            program.setStringField(stdin.readln("String field (optional)", null));

            if (stdin.readbool("Extract document properties", true, true))
            {
                while (pname == null)
                {
                    pname = stdin.readln("Phrase field for property names", "PROP_NAME");
                }
                program.setPNameField(pname);
                while (pval == null)
                {
                    pval = stdin.readln("Phrase field for property values", "PROP_VALUE");
                }
                program.setPValueField(pval);
            }

            // Perform the operation
            program.process();
        }
        catch (TdbException e)
        {
            System.out.println();
            System.out.println("[ERROR]");
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (program != null)
            {
                program.close();
                program = null;
            }
        }
    }
}
