///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2013 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//    TF Feb 18, 2013   Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.*;
import com.tietoenator.trip.jxp.data.*;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.database.TdbFieldDesign;
import com.tietoenator.trip.jxp.logging.*;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.util.ArrayList;

/**
 * TRIPjxp example program that shows how to perform an analysis of the
 * text in a record using TRIPcof and TRIPnlp to extract keywords and metadata
 * from text in existing records.
 * 
 * Note that this kind of analysis can be made also when adding a new record
 * to a database. It is also possible to perform text extraction via TRIPview,
 * TRIPview-C or TRIPcof during the same commit operation so that the extracted
 * text is immediately used as input to the TRIPcof/TRIPnlp text analysis.
 */
public class Nlp
{
   private TdbSession m_session;
   private TdbDatabaseDesign m_db;
   private TdbRecord m_rec;
   private TdbNlpInfo m_nlp;

   private int m_inputFieldCount = 0;
   private int m_outputFieldCount = 0;
   private ArrayList<String> m_allOutputFields = new ArrayList<String>();
   
   public Nlp() throws TdbException
   {
      // Create a session on a physical server (no grid support)
      m_session = SessionFactory.newInstance("**** TRIPnlp example ****", false);
   }
   
   private void setDatabase(String dbname)
           throws TdbException
   {
      m_db = new TdbDatabaseDesign(m_session);
      m_db.get(dbname);
   }
   
   private void setRecord(int rid)
           throws TdbException
   {
       m_rec = new TdbRecord(m_session, m_db, false);
       m_rec.setRecordId(rid);
       m_rec.get();
       m_nlp = m_rec.getNlpInfo();
   }

   private void addInputField(String fieldname)
           throws TdbException
   {
       m_nlp.addInputField(fieldname);
       m_inputFieldCount ++;
   }

   private int getInputFieldCount()
   {
       return m_inputFieldCount;
   }
   
   private void setKeywordField(String fieldname)
           throws TdbException
   {
       m_nlp.setKeywordField(fieldname);
       m_allOutputFields.add(fieldname);
       m_nlp.setExtractKeywords(true);
   }
   
   private void setMetadataOutputType(TdbNlpInfo.FieldMappingType mapType)
   {
       m_nlp.setFieldMapping(mapType);       
   }
   
   private void setTupledCategoryFields(String namefield, String valuefield)
           throws TdbException
   {
       m_nlp.setMetadataTupleFields(namefield, valuefield);
       m_allOutputFields.add(namefield);
       m_allOutputFields.add(valuefield);
       m_nlp.setExtractMetadata(true);
   }
 
   private void mapMetadataField(String category, String fieldname)
           throws TdbException
   {
       m_nlp.mapMetadataField(category, fieldname);
       m_allOutputFields.add(fieldname);
       m_outputFieldCount ++;
   }
   
   private int getOutputFieldCount()
   {
       return m_outputFieldCount;
   }
   
   private boolean isFieldOfType(String fieldname, TdbFieldType type)
   {
       TdbFieldDesign fd = m_db.getFieldByName(fieldname);
       return (fd != null && fd.getTypeId() == type) ? true : false;
   }
 
   private boolean isFieldOfTypes(String fieldname, TdbFieldType[] types)
   {
       TdbFieldDesign fd = m_db.getFieldByName(fieldname);
       for (TdbFieldType type : types)
       {
           if (type == fd.getTypeId())
               return true;
       }
       return false;
   }
   
   void process()
           throws TdbException
   {
       if (!m_nlp.isEnabled())
       {
           System.out.println("Insufficient number of parameters specified to NLP info object.");
           System.out.println("ANALYSIS FAILED");
           return;
       }

       // Turn on logging to show the XPI request and response messages
       m_session.setActivityLog(new TdbConsoleLogger(true));
       
       // We have already specified all required NLP parameters. 
       // Only thing left to do is to send the analysis request to the server.
       m_rec.commit();
       
       System.out.println("\nANALYSIS SUCCEEDED\n");
              
       // Retrieve the output fields for the record
       for (String fieldname : m_allOutputFields)
           m_rec.addToTemplate(fieldname);
       m_rec.get();

       System.out.println("Analysis results are:\n");
       
       TdbComponent h = m_rec.getHead();
       for (String fieldname : m_allOutputFields)
       {
           TdbField fld = h.getField(fieldname);
           System.out.print(fld.getName() + ": ");
           for (int i=0;i<fld.getValueCount();i++)
           {
               if (i>0)
                   System.out.print(", ");
               System.out.print(fld.getValue(i));
           }
           System.out.println("\n");
       }
   }
 
   /**
    * Main entry point
    * @param args Command line parameters (not used)
    */
   public static void main(String[] args)
   {
      try
      {
         Nlp prg = new Nlp();
         int rid;
         String db = null,  txt = null, kwfld = null, outputType = "T";
         boolean keywordAnalysis = false;
         boolean metadataAnalysis = false;
         TdbFieldType[] validinput = new TdbFieldType[]{TdbFieldType.PhraseField,TdbFieldType.TextField};
         TdbNlpInfo.FieldMappingType mapType = TdbNlpInfo.FieldMappingType.Tuples;
         
         Console stdin = new Console();

         // What database will we use?
         while (db == null)
         {
            db = stdin.readln("Database", null);
         }
         prg.setDatabase(db);
         
         do
         {
             rid = stdin.readint("Specify record id (RID) of record to process", 0, true);
         }
         while (rid < 1);
         prg.setRecord(rid);

         // Read a number of input fields
         System.out.println("Specify one or more input fields (end with a blank line).");
         do
         {
            txt = stdin.readln("Input field (text or phrase)", null);
            if (txt != null)
            {
                if (!prg.isFieldOfTypes(txt,validinput))
                {
                    System.out.println("Please specify a text or phrase field!");
                    txt = null;
                }
                else
                {
                    prg.addInputField(txt);
                }
            }
         }
         while (txt != null || prg.getInputFieldCount()==0);
         
         // Analysis type(s)
         do
         {
             keywordAnalysis = stdin.readbool("Keyword analysis", true, true);
             metadataAnalysis = stdin.readbool("Metadata analysis", true, true);
         
             if (!keywordAnalysis && !metadataAnalysis)
                System.out.println("Choose one or both of keyword and metadata analysis!");
         }
         while (!keywordAnalysis && !metadataAnalysis);
         
         // Keyword output field
         if (keywordAnalysis)
         {
             while (kwfld == null)
             {
                kwfld = stdin.readln("Keyword output field", null);
                
                if (kwfld != null)
                {
                    if (!prg.isFieldOfType(kwfld,TdbFieldType.PhraseField))
                    {
                        System.out.println("Please specify a phrase field!");
                        kwfld = null;
                    }
                    else
                        prg.setKeywordField(kwfld);
                }
             }
         }
         
         // Metdata output fields
         if (metadataAnalysis)
         {
             // Should we output to tuples (default) or to category-specific fields?
             do
             {
                 outputType = stdin.readln("Metadata output [T]tuples or [F]fields", "T");
             }
             while (outputType.equalsIgnoreCase("T")== false && outputType.equalsIgnoreCase("F")==false);
             
             mapType = outputType.equalsIgnoreCase("T") ? TdbNlpInfo.FieldMappingType.Tuples : TdbNlpInfo.FieldMappingType.Fields;
             prg.setMetadataOutputType(mapType);
             
             // Read tuple field names
             if (mapType == TdbNlpInfo.FieldMappingType.Tuples)
             {
                 String tn=null,tv=null;
                 do
                 {
                     tn = stdin.readln("Phrase field for category names",null);
                     if (tn != null && !prg.isFieldOfType(tn, TdbFieldType.PhraseField))
                     {
                         System.out.println("Please specify a phrase field!");
                         tn = null;
                     }
                 }
                 while (null == tn);
                 do
                 {
                     tv = stdin.readln("Phrase field for category values",null);
                     if (tv != null)
                     {
                        if (!prg.isFieldOfType(tv, TdbFieldType.PhraseField))
                        {
                            System.out.println("Please specify a phrase field!");
                            tv = null;
                        }
                        else if (tv.equalsIgnoreCase(tn))
                        {
                            System.out.println("Please specify a different phrase field!");
                            tv = null;
                        }
                     }
                 }
                 while (null == tv);
                 prg.setTupledCategoryFields(tn,tv);
             }
             
             // Read category/field pairs
             else
             {
                System.out.println("Specify one or more output fields (end with a blank category).");
                do
                {
                   String catname=null,catfield=null;
                   do
                   {
                       catname = stdin.readln("Category name", null);
                       if (catname == null && prg.getOutputFieldCount()>0)
                           break;
                   }
                   while (catname != null);

                    if (catname == null && prg.getOutputFieldCount()>0)
                        break;

                   do
                   {
                       catfield = stdin.readln("Field for category values", null);
                       if (catfield != null)
                       {
                          if (!prg.isFieldOfType(catfield,TdbFieldType.PhraseField))
                          {
                             System.out.println("Please specify a phrase field!");
                             catfield = null;
                          }
                          else
                          {
                             prg.mapMetadataField(catname, catfield);
                         }
                      }
                   }
                   while (catfield != null);
                }
                while (true);
             }
         }

         // Perform the analysis and display results
         prg.process();
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
         System.out.println("\nANALYSIS FAILED");
      }
      catch (Exception e)
      {
         e.printStackTrace();
         System.out.println("\nANALYSIS FAILED");         
      }
   }
}
