///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2019 infinIT Services GmbH
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//    TF  Apr 11, 2019  Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.session;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.session.TdbInterfaceType;
import com.tietoenator.trip.jxp.session.TdbTripNetSession;
import com.tietoenator.trip.jxp.session.TdbSession;

import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;


/**
 * Shows how to use the login ticket functionality in TRIP 8 to enable
 * an SSO-like behavior where a subsequent session can be initialized
 * without any action required by the end user.
 *
 * NOTE: for this to work, TRIPsystem 8.0 or later must be used and
 *       the login ticket feature must be enabled in tdbs.conf. The
 *       recommended minimum TRIPsysten version is 8.0-6 when using
 *       login tickets.
 */
public class LoginTicket
{
  private String host = null;
  private int port = 0;
  private boolean ticketUsed = false;
  private String oldTicket = null;
  
  LoginTicket() {}
  
  private boolean loginWithTicket(TdbSession session, String ticket)
  {
    try
    {
      session.login("",ticket);
      return true;
    }
    catch (TdbException x)
    {
      // The error message in the exception after a failed ticket login
      // is identical to that returned when neither username nor password
      // is supplied: "No access, username and password not entered."
      
      // Ticket login will also fail if the login ticket functionality
      // has been disabled on the server. In this case, the exception
      // message will be "Login ticket subsystem not enabled."
      
      // System.out.println(x.getDebugMessage());
      return false;
    }
  }
  
  private TdbSession login(String ticket)
  throws TdbException
  {
    boolean usernameRequired = false;
    String username, password = null;
    Console stdin = new Console();
    
    if (host == null || port < 1)
    {
      System.out.println("**** Login ticket example ****\n");
      
      host = stdin.readln("Server", "localhost");
      port = stdin.readint("Port#", 23457, true);
    }
    
    
    TdbSession session = null;
    try
    {
      session = new TdbTripNetSession(host, port, false);
    }
    catch (TdbException x)
    {
      if (session != null)
      {
        try { session.logout(); }
        catch (Exception ignore) {}
      }
      throw x;
    }
    
    if (ticket != null && ticket.length()>0)
    {
      if (loginWithTicket(session,ticket))
      {
        ticketUsed = true;
        return session;
      }
      else
      {
        // NOTE: This fallback can be tested by re-statrting tripd
        // after a successful ticket login and trying to use the ticket
        // again after restart. Another way to force this is to set the
        // ticket timeout (in tdbs.conf) to a very low value and wait
        // for this time to expire before trying again.
        
        System.out.println();
        System.out.println("FAILED to login using ticket!");
        System.out.println("Reverting to regular login behavior...");
        System.out.println();
        usernameRequired = true;
      }
    }
    
    
    // Collect username / password, login to the server
    if (usernameRequired)
    {
      // NOTE: The SYSTEM user cannot request login tickets.
      username = null;
      while (username == null)
      {
        username = stdin.readln("Username", null);
      }
    }
    else
    {
      username = stdin.readln("Username", "");
    }
    while (password == null)
    {
      password = stdin.readpw("Password", true);
    }
    
    
    if (username.length() == 0)
    {
      if (loginWithTicket(session,password))
      {
        // Informs this example app that a new ticket should not be obtained
        // since one was already entered on the command line.
        ticketUsed = true;
        oldTicket = password;
        return session;
      }
      else
      {
        System.out.println();
        System.out.println("ERROR: Invalid ticket. Please supply username and password.");
        System.out.println();
        
        username = null;
        while (username == null)
        {
          username = stdin.readln("Username", null);
        }
        password = null;
        while (password == null)
        {
          password = stdin.readpw("Password", true);
        }
      }
    }
    
    session.login(username,password);
    return session;
  }
  
  public static void main(String[] args)
  {
    String ticket;
    TdbSession session = null;
    LoginTicket prg = new LoginTicket();
    
    try
    {
      // The first login the user does is by specifying the username and password.
      //
      // NOTE: A ticket can be manually entered in this example by skipping the
      // and username and supplying the ticket in the password prompt.
      //
      session = prg.login("");
      System.out.println("Login 1: User \""+session.getUsername()+"\" logged in");
      
      // Request a login ticket.
      //
      // NOTE: If a login ticket already exists for the user, it will be replaced
      // by the new one.
      //
      // NOTE: The login ticket should be stored in a cookie or something similar
      // so that the web application can determine if the login form should
      // be displayed or if the login-ticket-based login should first be attempted.
      //
      if (prg.ticketUsed)
        ticket = prg.oldTicket;
      else
      {
        // If the login ticket functionality is not enabled on the server, an
        // exception will be thrown saying "Login ticket subsystem not enabled."
        //
        // Attempting to request a ticket for a user not allowed to use
        // tickets (such as SYSTEM) will cause an exception with the message
        // "Invalid username or password" to be thrown.
        //
        ticket = session.requestLoginTicket();
        
        // Display the ticket so that the scenario where the ticket is supplied
        // manually can be tested.
        System.out.println("TICKET: \""+ticket+"\"");
      }
      
      // Terminate the first session
      session.logout();
      session = null;
      
      
      // Start a new session using the login ticket
      session = prg.login(ticket);
      System.out.println("Login 2: User \""+session.getUsername()+"\" logged in");
      
      // Terminate the second session
      session.logout();
      session = null;
        
    
      
      // Start a new session using the login ticket, with the purpose of
      // revoking the ticket prematurely.
      session = prg.login(ticket);
      System.out.println("Login 3: User \""+session.getUsername()+"\" logged in");

      // NB: One can revoke the ticket by specifying it explicitly like this
      // or pass an empty string. Only the SYSTEM user may revoke the ticket
      // owned by another user.
      session.revokeLoginTicket(ticket);
      System.out.println("Ticket revoked");

    }
    catch (TdbException e)
    {
	  System.out.println(e.getDebugMessage());
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    finally
    {
      if (session != null)
      {
        try { session.logout(); }
        catch (Exception ignore) {}
      }
    }
  }
}

