///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp programming examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Aug 11, 2006	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.users;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;
import com.tietoenator.trip.jxp.users.TdbGroup;
import com.tietoenator.trip.jxp.users.TdbUser;

/**
 * Example of how to manage users and groups.
 */
public class UsersAndGroups
{

   private TdbSession m_session;

   // Simple utility function to print out whatever success messages have been
   // generated by a server transaction
   private void showMsgs(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
   }

   /**
    * Constructor
    * @throws TdbException
    */
   public UsersAndGroups()
           throws TdbException
   {
      m_session = SessionFactory.newInstance("**** User / Group Management Example ****", false);
   }

   /**
    * Run down the session
    * @throws TdbException
    */
   public void terminate()
           throws TdbException
   {
      m_session.logout();
   }

   /**
    * Create a new user
    * @param name Name of user to create
    * @param password Password of user to create
    * @throws TdbException
    */
   public void createUser(String name, String password)
           throws TdbException
   {
      TdbUser user = new TdbUser(m_session);
      user.create(name, password);
      showMsgs(user);
   }

   /**
    * Create a new group
    * @param name Name of the group to create
    * @throws TdbException
    */
   public void createGroup(String name)
           throws TdbException
   {
      TdbGroup group = new TdbGroup(m_session);
      group.create(name);
      showMsgs(group);
   }

   /**
    * Modify the properties of a user
    * @param userName Name of user
    * @throws TdbException
    */
   public void modifyUser(String userName)
           throws TdbException
   {
      TdbUser user = new TdbUser(m_session);
      boolean quit = false, save = false;
      Console stdin = new Console();
      String str;

      user.get(userName);
      for (; !quit && !save;)
      {
         System.out.println("\n0) Quit without saving.");
         System.out.println("1) Save and exit.");
         System.out.println("2) Modify real name.");
         System.out.println("3) Modify login procedure.");
         System.out.println("4) Modify date format.");
         int val = stdin.readint("\nOption", 1, true);
         switch (val)
         {
         case 0:
            quit = true;
            break;
         case 1:
            save = true;
            break;
         case 2:
            str = user.getRealName();
            if (str.length() > 0)
            {
               System.out.println("Current value: " + str);
            }
            user.setRealName(stdin.readln("New value", ""));
            break;

         case 3:
            str = user.getLoginProcedure();
            if (str.length() > 0)
            {
               System.out.println("Current value: " + str);
            }
            user.setLoginProcedure(stdin.readln("New value", ""));
            break;

         case 4:
            System.out.println("Current dateform: " + m_session.getDateform(user.getDateform()));
            int ix = 1;
            for (String df : m_session.dateforms())
            {
               System.out.println("" + ix++ + ") " + df);
            }
            val = stdin.readint("New value", 1, true);
            if (val < 0 || val >= ix)
            {
               System.out.println("Invalid choice.");
            }
            else
            {
               user.setDateform(val - 1);
            }
            break;
         }
      }
      if (!quit)
      {
         user.put();
         showMsgs(user);
      }
   }

   /**
    * Adds a user to a group
    * @param groupName Group name
    * @param userName User name
    * @throws TdbException
    */
   public void addUserToGroup(String groupName, String userName)
           throws TdbException
   {
      TdbGroup group = new TdbGroup(m_session);
      group.add(userName, groupName);
      showMsgs(group);
   }

   /**
    * Removes a user from an existing group
    * @param groupName Group from which the user is to be removed
    * @param userName User to be removed from the group
    * @throws TdbException
    */
   public void removeUserFromGroup(String groupName, String userName)
           throws TdbException
   {
      TdbGroup group = new TdbGroup(m_session);
      group.remove(userName, groupName);
      showMsgs(group);
   }

   /**
    * Remove an existing user
    * @param name Name of user to delete
    * @throws TdbException
    */
   public void removeUser(String name)
           throws TdbException
   {
      TdbUser user = new TdbUser(m_session);
      user.delete(name);
      showMsgs(user);
   }

   /**
    * Remove an existing user group
    * @param name Name of the group to delete
    * @throws TdbException
    */
   public void removeGroup(String name)
           throws TdbException
   {
      TdbGroup group = new TdbGroup(m_session);
      group.delete(name);
      showMsgs(group);
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         UsersAndGroups ug = new UsersAndGroups();
         Console stdin = new Console();
         boolean quit = false;
         String user = "", password = "", group = "";

         for (; !quit;)
         {
            try
            {
               System.out.println("\n0) Quit (default)");
               System.out.println("1) Create new user.");
               System.out.println("2) Modify existing user.");
               System.out.println("3) Remove existing user.");
               System.out.println("4) Create new group.");
               System.out.println("5) Add user to group.");
               System.out.println("6) Remove user from group.");
               System.out.println("7) Remove group.");
               switch (stdin.readint("\nChoose function", 0, true))
               {
               case 0:
                  quit = true;
                  break;
               case 1:
                  user = stdin.readln("New user's name", "");
                  if (user.length() > 0)
                  {
                     password = stdin.readln("New user's password", "");
                     ug.createUser(user, password);
                  }
                  break;

               case 2:
                  user = stdin.readln("User's name", user);
                  if (user.length() > 0)
                  {
                     ug.modifyUser(user);
                  }
                  break;

               case 3:
                  user = stdin.readln("User's name", user);
                  if (user.length() > 0)
                  {
                     ug.removeUser(user);
                  }
                  break;

               case 4:
                  group = stdin.readln("Group name", group);
                  if (group.length() > 0)
                  {
                     ug.createGroup(group);
                  }
                  break;

               case 5:
                  group = stdin.readln("Group name", group);
                  if (group.length() > 0)
                  {
                     user = stdin.readln("User's name", user);
                     if (user.length() > 0)
                     {
                        ug.addUserToGroup(group, user);
                     }
                  }
                  break;

               case 6:
                  group = stdin.readln("Group name", group);
                  if (group.length() > 0)
                  {
                     user = stdin.readln("User's name", user);
                     if (user.length() > 0)
                     {
                        ug.addUserToGroup(group, user);
                     }
                  }
                  break;

               case 7:
                  group = stdin.readln("Group name", group);
                  if (group.length() > 0)
                  {
                     ug.removeGroup(group);
                  }
                  break;
               }
            }
            catch (Exception e)
            {
               System.out.println(e.getMessage());
            }
         }
         ug.terminate();
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
      }
   }
}
